@@ -23,11 +23,14 @@ import (
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"k8s.io/apimachinery/pkg/runtime"
+	"k8s.io/apimachinery/pkg/runtime/schema"
 	k8sjson "k8s.io/apimachinery/pkg/runtime/serializer/json"
 	"k8s.io/apimachinery/pkg/util/diff"
 	"k8s.io/apimachinery/pkg/util/wait"
 	"k8s.io/apiserver/pkg/storage/names"
+	"k8s.io/client-go/dynamic"
 	k8sscheme "k8s.io/client-go/kubernetes/scheme"
+	"k8s.io/client-go/rest"
 
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1alpha1"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/client"
@@ -105,6 +108,13 @@ func newCRClient(t *testing.T) versioned.Interface {
 	return crclient
 }
 
+func newDynamicClient(t *testing.T, config *rest.Config) dynamic.Interface {
+	// TODO: impersonate ALM serviceaccount
+	dynamicClient, err := dynamic.NewForConfig(config)
+	require.NoError(t, err)
+	return dynamicClient
+}
+
 func newPMClient(t *testing.T) pmversioned.Interface {
 	if kubeConfigPath == nil {
 		t.Log("using in-cluster config")
@@ -138,6 +148,29 @@ func awaitPods(t *testing.T, c operatorclient.ClientInterface, namespace, select
 	return fetchedPodList, err
 }
 
+func awaitPodsWithInterval(t *testing.T, c operatorclient.ClientInterface, namespace, selector string, interval time.Duration,
+	duration time.Duration, checkPods podsCheckFunc) (*corev1.PodList, error) {
+	var fetchedPodList *corev1.PodList
+	var err error
+
+	err = wait.Poll(interval, duration, func() (bool, error) {
+		fetchedPodList, err = c.KubernetesInterface().CoreV1().Pods(namespace).List(metav1.ListOptions{
+			LabelSelector: selector,
+		})
+
+		if err != nil {
+			return false, err
+		}
+
+		t.Logf("Waiting for pods matching selector %s to match given conditions", selector)
+
+		return checkPods(fetchedPodList), nil
+	})
+
+	require.NoError(t, err)
+	return fetchedPodList, err
+}
+
 // podsCheckFunc describes a function that true if the given PodList meets some criteria; false otherwise.
 type podsCheckFunc func(pods *corev1.PodList) bool
 
@@ -281,6 +314,19 @@ func waitForEmptyList(checkList func() (int, error)) error {
 	return err
 }
 
+func waitForGVR(dynamicClient dynamic.Interface, gvr schema.GroupVersionResource, name, namespace string) error {
+	return wait.Poll(pollInterval, pollDuration, func() (bool, error) {
+		_, err := dynamicClient.Resource(gvr).Namespace(namespace).Get(name, metav1.GetOptions{})
+		if err != nil {
+			if errors.IsNotFound(err) {
+				return false, nil
+			}
+			return false, err
+		}
+		return true, nil
+	})
+}
+
 type catalogSourceCheckFunc func(*v1alpha1.CatalogSource) bool
 
 // This check is disabled for most test runs, but can be enabled for verifying pod health if the e2e tests are running
@@ -554,3 +600,28 @@ func buildCRCleanupFunc(c operatorclient.ClientInterface, apiGroup, version, nam
 		})
 	}
 }
+
+// Local determines whether test is running locally or in a container on openshift-CI.
+// Queries for a clusterversion object specific to OpenShift.
+func Local(client operatorclient.ClientInterface) (bool, error) {
+	const ClusterVersionGroup = "config.openshift.io"
+	const ClusterVersionVersion = "v1"
+	const ClusterVersionKind = "ClusterVersion"
+	gv := metav1.GroupVersion{Group: ClusterVersionGroup, Version: ClusterVersionVersion}.String()
+
+	groups, err := client.KubernetesInterface().Discovery().ServerResourcesForGroupVersion(gv)
+	if err != nil {
+		if errors.IsNotFound(err) {
+			return true, nil
+		}
+		return true, fmt.Errorf("checking if cluster is local: checking server groups: %s", err)
+	}
+
+	for _, group := range groups.APIResources {
+		if group.Kind == ClusterVersionKind {
+			return false, nil
+		}
+	}
+
+	return true, nil
+}
\ No newline at end of file
